
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/Manageable.h"
#include "qpid/management/Buffer.h"
#include "qpid/types/Variant.h"
#include "qpid/amqp_0_10/Codecs.h"
#include "qpid/agent/ManagementAgent.h"
#include "Submission.h"
#include "ArgsGridMethodsEcho.h"
#include "ArgsSubmissionGetJobSummaries.h"

#include <iostream>
#include <sstream>

using namespace qmf::com::redhat::grid;
using           qpid::management::ManagementAgent;
using           qpid::management::Manageable;
using           qpid::management::ManagementObject;
using           qpid::management::Args;
using           qpid::management::Mutex;
using           std::string;

string  Submission::packageName  = string ("com.redhat.grid");
string  Submission::className    = string ("submission");
uint8_t Submission::md5Sum[MD5_LEN]   =
    {0x34,0xc7,0x45,0xf2,0xfe,0xbe,0x1d,0xc8,0x53,0xce,0x61,0x42,0xc3,0xb2,0xbd,0xb8};

Submission::Submission (ManagementAgent*, Manageable* _core, ::qpid::management::Manageable* _parent) :
    ManagementObject(_core)
{
    jobserverRef = _parent->GetManagementObject ()->getObjectId ();
    Name = "";
    Owner = "";
    QDate = 0;



    perThreadStatsArray = new struct PerThreadStats*[maxThreads];
    for (int idx = 0; idx < maxThreads; idx++)
        perThreadStatsArray[idx] = 0;

}

Submission::~Submission ()
{

    for (int idx = 0; idx < maxThreads; idx++)
        if (perThreadStatsArray[idx] != 0)
            delete perThreadStatsArray[idx];
    delete[] perThreadStatsArray;

}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string IS_INDEX("index");
    const string IS_OPTIONAL("optional");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Submission::registerSelf(ManagementAgent* agent)
{
    agent->registerClass(packageName, className, md5Sum, writeSchema);
}

void Submission::writeSchema (std::string& schema)
{
    const int _bufSize=65536;
    char _msgChars[_bufSize];
    ::qpid::management::Buffer buf(_msgChars, _bufSize);
    ::qpid::types::Variant::Map ft;

    // Schema class header:
    buf.putOctet       (CLASS_KIND_TABLE);
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (4); // Config Element Count
    buf.putShort       (5); // Inst Element Count
    buf.putShort       (2); // Method Count

    // Properties
    ft.clear();
    ft[NAME] = "jobserverRef";
    ft[TYPE] = TYPE_REF;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 1;
    ft[IS_OPTIONAL] = 0;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Name";
    ft[TYPE] = TYPE_SSTR;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 1;
    ft[IS_OPTIONAL] = 0;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Owner";
    ft[TYPE] = TYPE_SSTR;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 1;
    ft[IS_OPTIONAL] = 0;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "QDate";
    ft[TYPE] = TYPE_ABSTIME;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 0;
    ft[IS_OPTIONAL] = 0;
    ft[DESC] = "Oldest QDate of member jobs";
    buf.putMap(ft);


    // Statistics
    ft.clear();
    ft[NAME] = "Idle";
    ft[TYPE] = TYPE_U32;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Running";
    ft[TYPE] = TYPE_U32;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Removed";
    ft[TYPE] = TYPE_U32;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Completed";
    ft[TYPE] = TYPE_U32;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Held";
    ft[TYPE] = TYPE_U32;
    buf.putMap(ft);


    // Methods
    ft.clear();
    ft[NAME] =  "echo";
    ft[ARGCOUNT] = 2;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "sequence";
    ft[TYPE] = TYPE_U32;
    ft[DIR] = "IO";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "body";
    ft[TYPE] = TYPE_LSTR;
    ft[DIR] = "IO";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] =  "GetJobSummaries";
    ft[ARGCOUNT] = 1;
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "Jobs";
    ft[TYPE] = TYPE_LIST;
    ft[DIR] = "O";
    buf.putMap(ft);


    {
        uint32_t _len = buf.getPosition();
        buf.reset();
        buf.getRawData(schema, _len);
    }
}


void Submission::aggregatePerThreadStats(struct PerThreadStats* totals) const
{
    totals->Idle = 0;
    totals->Running = 0;
    totals->Removed = 0;
    totals->Completed = 0;
    totals->Held = 0;

    for (int idx = 0; idx < maxThreads; idx++) {
        struct PerThreadStats* threadStats = perThreadStatsArray[idx];
        if (threadStats != 0) {
            totals->Idle += threadStats->Idle;
            totals->Running += threadStats->Running;
            totals->Removed += threadStats->Removed;
            totals->Completed += threadStats->Completed;
            totals->Held += threadStats->Held;

        }
    }
}



std::string Submission::getKey() const
{
    std::stringstream key;

    key << jobserverRef.getV2Key() << ",";
    key << Name << ",";
    key << Owner;
    return key.str();
}



void Submission::mapEncodeValues (::qpid::types::Variant::Map& _map,
                                              bool includeProperties,
                                              bool includeStatistics)
{
    using namespace ::qpid::types;
    Mutex::ScopedLock mutex(accessLock);

    if (includeProperties) {
        configChanged = false;
    _map["jobserverRef"] = ::qpid::types::Variant(jobserverRef);
    _map["Name"] = ::qpid::types::Variant(Name);
    _map["Owner"] = ::qpid::types::Variant(Owner);
    _map["QDate"] = ::qpid::types::Variant(QDate);

    }

    if (includeStatistics) {
        instChanged = false;


        struct PerThreadStats totals;
        aggregatePerThreadStats(&totals);



    _map["Idle"] = ::qpid::types::Variant(totals.Idle);
    _map["Running"] = ::qpid::types::Variant(totals.Running);
    _map["Removed"] = ::qpid::types::Variant(totals.Removed);
    _map["Completed"] = ::qpid::types::Variant(totals.Completed);
    _map["Held"] = ::qpid::types::Variant(totals.Held);


    // Maintenance of hi-lo statistics


    }
}

void Submission::mapDecodeValues (const ::qpid::types::Variant::Map& _map)
{
    ::qpid::types::Variant::Map::const_iterator _i;
    Mutex::ScopedLock mutex(accessLock);

    if ((_i = _map.find("jobserverRef")) != _map.end()) {
        jobserverRef = _i->second;
    }
    if ((_i = _map.find("Name")) != _map.end()) {
        Name = (_i->second).getString();
    }
    if ((_i = _map.find("Owner")) != _map.end()) {
        Owner = (_i->second).getString();
    }
    if ((_i = _map.find("QDate")) != _map.end()) {
        QDate = _i->second;
    }

}

void Submission::doMethod (string& methodName, const ::qpid::types::Variant::Map& inMap, ::qpid::types::Variant::Map& outMap, const string& userId)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;
    std::string          text;


    if (methodName == "echo") {
        ArgsGridMethodsEcho ioArgs;
        ::qpid::types::Variant::Map::const_iterator _i;
        if ((_i = inMap.find("sequence")) != inMap.end()) {
            ioArgs.io_sequence = _i->second;
        }
        if ((_i = inMap.find("body")) != inMap.end()) {
            ioArgs.io_body = (_i->second).getString();
        }
        bool allow = coreObject->AuthorizeMethod(METHOD_ECHO, ioArgs, userId);
        if (allow)
            status = coreObject->ManagementMethod (METHOD_ECHO, ioArgs, text);
        else
            status = Manageable::STATUS_FORBIDDEN;
        outMap["_status_code"] = (uint32_t) status;
        outMap["_status_text"] = ::qpid::management::Manageable::StatusText(status, text);
        outMap["sequence"] = ::qpid::types::Variant(ioArgs.io_sequence);
        outMap["body"] = ::qpid::types::Variant(ioArgs.io_body);
        return;
    }

    if (methodName == "GetJobSummaries") {
        ArgsSubmissionGetJobSummaries ioArgs;
        ::qpid::types::Variant::Map::const_iterator _i;
        bool allow = coreObject->AuthorizeMethod(METHOD_GETJOBSUMMARIES, ioArgs, userId);
        if (allow)
            status = coreObject->ManagementMethod (METHOD_GETJOBSUMMARIES, ioArgs, text);
        else
            status = Manageable::STATUS_FORBIDDEN;
        outMap["_status_code"] = (uint32_t) status;
        outMap["_status_text"] = ::qpid::management::Manageable::StatusText(status, text);
        outMap["Jobs"] = ::qpid::types::Variant(ioArgs.o_Jobs);
        return;
    }

    outMap["_status_code"] = (uint32_t) status;
    outMap["_status_text"] = Manageable::StatusText(status, text);
}
